@@ -1,6 +1,6 @@
 /* This module implements the relp sess object.
  *
- * Copyright 2008-2018 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2008-2020 by Rainer Gerhards and Adiscon GmbH.
  *
  * To clarify on session handling: There is no upper limit of the number
  * of sessions, except for system ressources. Some left-over comment
@@ -57,9 +57,9 @@
 #include "dbllinklist.h"
 
 /* forward definitions */
-static relpRetVal relpSessCltDoDisconnect(relpSess_t *pThis);
-static relpRetVal relpSessFixCmdStates(relpSess_t *pThis);
-static relpRetVal relpSessSrvDoDisconnect(relpSess_t *pThis);
+static relpRetVal relpSessCltDoDisconnect(relpSess_t *const pThis);
+static relpRetVal relpSessFixCmdStates(relpSess_t *const pThis);
+static relpRetVal relpSessSrvDoDisconnect(relpSess_t *const pThis);
 
 /* helper to call onErr if set */
 static void
@@ -68,7 +68,7 @@ callOnErr(const relpSess_t *__restrict__ const pThis,
 	const relpRetVal ecode)
 {
 	relpTcp_t *const pTcp = pThis->pTcp;
-	//pThis->pEngine->dbgprint("librelp: generic error: ecode %d, "
+	//pThis->pEngine->dbgprint((char*)"librelp: generic error: ecode %d, "
 		//"emsg '%s'\n", ecode, emsg);
 	if(pThis->pEngine->onErr != NULL) {
 		char objinfo[1024];
@@ -92,7 +92,7 @@ callOnErr(const relpSess_t *__restrict__ const pThis,
 
 /* helper to free permittedPeer structure */
 static inline void
-relpSessFreePermittedPeers(relpSess_t *pThis)
+relpSessFreePermittedPeers(relpSess_t *const pThis)
 {
 	int i;
 	for(i = 0 ; i < pThis->permittedPeers.nmemb ; ++i)
@@ -224,7 +224,7 @@ relpSessDestruct(relpSess_t **ppThis)
  * rgerhards, 2008-03-17
  */
 relpRetVal
-relpSessAcceptAndConstruct(relpSess_t **ppThis, relpSrv_t *pSrv, int sock)
+relpSessAcceptAndConstruct(relpSess_t **ppThis, relpSrv_t *const pSrv, const int sock)
 {
 	relpSess_t *pThis = NULL;
 
@@ -257,7 +257,7 @@ finalize_it:
  * rgerhards, 2008-03-17
  */
 relpRetVal
-relpSessRcvData(relpSess_t *pThis)
+relpSessRcvData(relpSess_t *const pThis)
 {
 	relpOctet_t rcvBuf[RELP_RCV_BUF_SIZE+1];
 	ssize_t lenBuf;
@@ -270,8 +270,8 @@ relpSessRcvData(relpSess_t *pThis)
 	CHKRet(relpTcpRcv(pThis->pTcp, rcvBuf, &lenBuf));
 
 	if(lenBuf == 0) {
-		callOnErr(pThis, "server closed relp session, session broken", RELP_RET_SESSION_BROKEN);
-		pThis->pEngine->dbgprint("server closed relp session %p, session broken\n", (void*)pThis);
+		callOnErr(pThis, (char*) "server closed relp session, session broken", RELP_RET_SESSION_BROKEN);
+		pThis->pEngine->dbgprint((char*)"server closed relp session %p, session broken\n", (void*)pThis);
 		/* even though we had a "normal" close, it is unexpected at this
 		 * stage. Consequently, we consider the session to be broken, because
 		 * the recovery action is the same no matter how it is broken.
@@ -279,9 +279,9 @@ relpSessRcvData(relpSess_t *pThis)
 		pThis->sessState = eRelpSessState_BROKEN;
 		ABORT_FINALIZE(RELP_RET_SESSION_BROKEN);
 	} else if ((int) lenBuf == -1) {
-		if(errno != EAGAIN) {
-			callOnErr(pThis, "error when receiving data, session broken", RELP_RET_SESSION_BROKEN);
-			pThis->pEngine->dbgprint("errno %d during relp session %p, session broken\n",
+		if(pThis->pTcp->rtryOp != relpTCP_RETRY_recv) {
+			callOnErr(pThis, (char*) "error when receiving data, session broken", RELP_RET_SESSION_BROKEN);
+			pThis->pEngine->dbgprint((char*)"errno %d during relp session %p, session broken\n",
 				errno, (void*)pThis);
 			pThis->sessState = eRelpSessState_BROKEN;
 			ABORT_FINALIZE(RELP_RET_SESSION_BROKEN);
@@ -289,12 +289,12 @@ relpSessRcvData(relpSess_t *pThis)
 	} else {
 		/* Terminate buffer and output received data to debug*/
 		rcvBuf[lenBuf] = '\0';
-		pThis->pEngine->dbgprint("relp session read %d octets, buf '%s'\n", (int) lenBuf, rcvBuf);
+		pThis->pEngine->dbgprint((char*)"relp session read %d octets, buf '%s'\n", (int) lenBuf, rcvBuf);
 
 		/* we have regular data, which we now can process */
 		for(i = 0 ; i < lenBuf ; ++i) {
 			if(relpEngineShouldStop(pThis->pEngine)) {
-				pThis->pEngine->dbgprint("imrelp is instructed to shut down, thus "
+				pThis->pEngine->dbgprint((char*)"imrelp is instructed to shut down, thus "
 					"breaking session %p\n", (void*) pThis);
 				ABORT_FINALIZE(RELP_RET_SESSION_BROKEN);
 			}
@@ -311,7 +311,7 @@ finalize_it:
  * rgerhards, 2008-03-19
  */
 relpRetVal
-relpSessSendResponse(relpSess_t *pThis, relpTxnr_t txnr, unsigned char *pData, size_t lenData)
+relpSessSendResponse(relpSess_t *const pThis, relpTxnr_t txnr, unsigned char *pData, size_t lenData)
 {
 	relpSendbuf_t *pSendbuf;
 
@@ -326,13 +326,18 @@ relpSessSendResponse(relpSess_t *pThis, relpTxnr_t txnr, unsigned char *pData, s
 finalize_it:
 	if(iRet != RELP_RET_OK) {
 		if(iRet == RELP_RET_IO_ERR) {
-			callOnErr(pThis, "io error, session broken", RELP_RET_SESSION_BROKEN);
-			pThis->pEngine->dbgprint("relp session %p is broken, io error\n", (void*)pThis);
+			callOnErr(pThis, (char*) "io error, session broken", RELP_RET_SESSION_BROKEN);
+			pThis->pEngine->dbgprint((char*)"relp session %p is broken, io error\n", (void*)pThis);
 			pThis->sessState = eRelpSessState_BROKEN;
-			}
+		} else {
+			/*	alorbach, 2020-04-08:
+			*		Only free sendbuf if error is not RELP_RET_IO_ERR!
+			*		otherwise the buffer is double freed in relpSendbufDestruct() (sendbuf.c)
+			*/
+			if(pSendbuf != NULL)
+				relpSendbufDestruct(&pSendbuf);
+		}
 
-		if(pSendbuf != NULL)
-			relpSendbufDestruct(&pSendbuf);
 	}
 
 	LEAVE_RELPFUNC;
@@ -343,7 +348,7 @@ finalize_it:
  * rgerhards, 2008-03-19
  */
 relpRetVal
-relpSessSndData(relpSess_t *pThis)
+relpSessSndData(relpSess_t *const pThis)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -365,7 +370,7 @@ finalize_it:
  * rgerhards, 2008-03-31
  */
 static relpRetVal
-relpSessSrvSendHint(relpSess_t *pThis, unsigned char *pHint, size_t lenHint,
+relpSessSrvSendHint(relpSess_t *const pThis, unsigned char *pHint, size_t lenHint,
 		    unsigned char *pData, size_t lenData)
 {
 	relpSendbuf_t *pSendbuf = NULL;
@@ -377,7 +382,7 @@ relpSessSrvSendHint(relpSess_t *pThis, unsigned char *pHint, size_t lenHint,
 
 	CHKRet(relpFrameBuildSendbuf(&pSendbuf, 0, pHint, lenHint, pData, lenData, pThis, NULL));
 	/* now send it */
-	pThis->pEngine->dbgprint("hint-frame to send: '%s'\n", pSendbuf->pData + (9 - pSendbuf->lenTxnr));
+	pThis->pEngine->dbgprint((char*)"hint-frame to send: '%s'\n", pSendbuf->pData + (9 - pSendbuf->lenTxnr));
 	CHKRet(relpSendbufSend(pSendbuf, pThis->pTcp));
 
 finalize_it:
@@ -393,7 +398,7 @@ finalize_it:
  * rgerhards, 2008-03-31
  */
 static relpRetVal
-relpSessSrvDoDisconnect(relpSess_t *pThis)
+relpSessSrvDoDisconnect(relpSess_t *const pThis)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -421,7 +426,7 @@ finalize_it:
  * rgerhards, 2008-03-20
  */
 relpRetVal
-relpSessAddUnacked(relpSess_t *pThis, relpSendbuf_t *pSendbuf)
+relpSessAddUnacked(relpSess_t *const pThis, relpSendbuf_t *pSendbuf)
 {
 	relpSessUnacked_t *pUnackedLstEntry;
 
@@ -445,10 +450,10 @@ relpSessAddUnacked(relpSess_t *pThis, relpSendbuf_t *pSendbuf)
 		 */
 		relpSessSetSessState(pThis, eRelpSessState_WINDOW_FULL);
 		if(pThis->lenUnackedLst >= pThis->sizeWindow)
-			pThis->pEngine->dbgprint("Warning: exceeding window size, max %d, curr %d\n",
+			pThis->pEngine->dbgprint((char*)"Warning: exceeding window size, max %d, curr %d\n",
 						 pThis->lenUnackedLst, pThis->sizeWindow);
 	}
-	pThis->pEngine->dbgprint("ADD sess %p unacked %d, sessState %d\n",
+	pThis->pEngine->dbgprint((char*)"ADD sess %p unacked %d, sessState %d\n",
 		(void*)pThis, pThis->lenUnackedLst, pThis->sessState);
 
 finalize_it:
@@ -461,7 +466,7 @@ finalize_it:
  * rgerhards, 2008-03-20
  */
 static relpRetVal
-relpSessDelUnacked(relpSess_t *pThis, relpSessUnacked_t *pUnackedLstEntry)
+relpSessDelUnacked(relpSess_t *const pThis, relpSessUnacked_t *pUnackedLstEntry)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -482,7 +487,7 @@ relpSessDelUnacked(relpSess_t *pThis, relpSessUnacked_t *pUnackedLstEntry)
 
 	free(pUnackedLstEntry);
 
-	pThis->pEngine->dbgprint("DEL sess %p unacked %d, sessState %d\n",
+	pThis->pEngine->dbgprint((char*)"DEL sess %p unacked %d, sessState %d\n",
 		(void*)pThis, pThis->lenUnackedLst, pThis->sessState);
 	LEAVE_RELPFUNC;
 }
@@ -494,7 +499,7 @@ relpSessDelUnacked(relpSess_t *pThis, relpSessUnacked_t *pUnackedLstEntry)
  * rgerhards, 20080-03-20
  */
 relpRetVal
-relpSessGetUnacked(relpSess_t *pThis, relpSendbuf_t **ppSendbuf, relpTxnr_t txnr)
+relpSessGetUnacked(relpSess_t *const pThis, relpSendbuf_t **ppSendbuf, relpTxnr_t txnr)
 {
 	relpSessUnacked_t *pUnackedEtry;
 
@@ -549,7 +554,7 @@ relpSessWaitState(relpSess_t *const pThis, const relpSessState_t stateExpected,
 	 */
 	localRet = relpSessRcvData(pThis);
 	if(localRet != RELP_RET_OK) {
-		pThis->pEngine->dbgprint("relpSessWaitRsp error initial rcv: %d\n", localRet);
+		pThis->pEngine->dbgprint((char*)"relpSessWaitRsp error initial rcv: %d\n", localRet);
 		ABORT_FINALIZE(localRet);
 	}
 
@@ -575,25 +580,25 @@ relpSessWaitState(relpSess_t *const pThis, const relpSessState_t stateExpected,
 
 		pfd.fd = sock;
 		pfd.events = POLLIN;
-		pThis->pEngine->dbgprint("relpSessWaitRsp waiting for data on "
+		pThis->pEngine->dbgprint((char*)"relpSessWaitRsp waiting for data on "
 			"fd %d, timeout %d, state expected %d\n", sock, timeout, stateExpected);
 		nfds = poll(&pfd, 1, timeout*1000);
 		if(nfds == -1) {
 			if(errno == EINTR) {
-				pThis->pEngine->dbgprint("relpSessWaitRsp select interrupted, continue\n");
+				pThis->pEngine->dbgprint((char*)"relpSessWaitRsp select interrupted, continue\n");
 			} else {
-				pThis->pEngine->dbgprint("relpSessWaitRsp select returned error %d\n", errno);
+				pThis->pEngine->dbgprint((char*)"relpSessWaitRsp select returned error %d\n", errno);
 				ABORT_FINALIZE(RELP_RET_SESSION_BROKEN);
 			}
 		}
 		else
-			pThis->pEngine->dbgprint("relpSessWaitRsp poll returns, "
+			pThis->pEngine->dbgprint((char*)"relpSessWaitRsp poll returns, "
 				"nfds %d, errno %d\n", nfds, errno);
 		if(relpEngineShouldStop(pThis->pEngine))
 			break;
 		/* we don't check if we had a timeout-we give it one last chance*/
 		CHKRet(relpSessRcvData(pThis));
-		pThis->pEngine->dbgprint("iRet after relpSessRcvData %d\n", iRet);
+		pThis->pEngine->dbgprint((char*)"iRet after relpSessRcvData %d\n", iRet);
 		if(   pThis->sessState == stateExpected
 		   || pThis->sessState == eRelpSessState_BROKEN) {
 			FINALIZE;
@@ -603,12 +608,12 @@ relpSessWaitState(relpSess_t *const pThis, const relpSessState_t stateExpected,
 	}
 
 finalize_it:
-	pThis->pEngine->dbgprint("relpSessWaitState returns %d\n", iRet);
+	pThis->pEngine->dbgprint((char*)"relpSessWaitState returns %d\n", iRet);
 	if(	iRet == RELP_RET_TIMED_OUT ||
 		iRet == RELP_RET_SESSION_BROKEN ||
 		relpEngineShouldStop(pThis->pEngine)) {
 		/* the session is broken! */
-		callOnErr(pThis, "error waiting on required session state, session broken",
+		callOnErr(pThis, (char*) "error waiting on required session state, session broken",
 			RELP_RET_SESSION_BROKEN);
 		pThis->sessState = eRelpSessState_BROKEN;
 	}
@@ -625,7 +630,7 @@ finalize_it:
  * rgerhards, 2008-03-19
  */
 static relpRetVal
-relpSessRawSendCommand(relpSess_t *pThis, unsigned char *pCmd, size_t lenCmd,
+relpSessRawSendCommand(relpSess_t *const pThis, unsigned char *pCmd, size_t lenCmd,
 		    unsigned char *pData, size_t lenData, relpRetVal (*rspHdlr)(relpSess_t*,relpFrame_t*))
 {
 	relpSendbuf_t *pSendbuf;
@@ -636,12 +641,12 @@ relpSessRawSendCommand(relpSess_t *pThis, unsigned char *pCmd, size_t lenCmd,
 	CHKRet(relpFrameBuildSendbuf(&pSendbuf, pThis->txnr, pCmd, lenCmd, pData, lenData, pThis, rspHdlr));
 	pThis->txnr = relpEngineNextTXNR(pThis->txnr);
 	/* now send it */
-	pThis->pEngine->dbgprint("frame to send: '%s'\n", pSendbuf->pData + (9 - pSendbuf->lenTxnr));
+	pThis->pEngine->dbgprint((char*)"frame to send: '%s'\n", pSendbuf->pData + (9 - pSendbuf->lenTxnr));
 	iRet = relpSendbufSendAll(pSendbuf, pThis, 1);
 
 	if(iRet == RELP_RET_IO_ERR) {
-		pThis->pEngine->dbgprint("relp session %p flagged as broken, IO error\n", (void*)pThis);
-		callOnErr(pThis, "io error in RawSendCommand, session broken", RELP_RET_SESSION_BROKEN);
+		pThis->pEngine->dbgprint((char*)"relp session %p flagged as broken, IO error\n", (void*)pThis);
+		callOnErr(pThis, (char*) "io error in RawSendCommand, session broken", RELP_RET_SESSION_BROKEN);
 		pThis->sessState = eRelpSessState_BROKEN;
 		ABORT_FINALIZE(RELP_RET_SESSION_BROKEN);
 	}
@@ -659,7 +664,7 @@ finalize_it:
  * rgerhards, 2008-03-19
  */
 relpRetVal
-relpSessSendCommand(relpSess_t *pThis, unsigned char *pCmd, size_t lenCmd,
+relpSessSendCommand(relpSess_t *const pThis, unsigned char *pCmd, size_t lenCmd,
 		    unsigned char *pData, size_t lenData, relpRetVal (*rspHdlr)(relpSess_t*,relpFrame_t*))
 {
 	ENTER_RELPFUNC;
@@ -695,7 +700,7 @@ finalize_it:
  * rgerhards, 2008-03-22
  */
 relpRetVal
-relpSessTryReestablish(relpSess_t *pThis)
+relpSessTryReestablish(relpSess_t *const pThis)
 {
 	relpSessUnacked_t *pUnackedEtry;
 
@@ -713,12 +718,12 @@ relpSessTryReestablish(relpSess_t *pThis)
 	 */
 	pUnackedEtry = pThis->pUnackedLstRoot;
 	if(pUnackedEtry != NULL)
-		pThis->pEngine->dbgprint("relp session %p reestablished, state %d, "
+		pThis->pEngine->dbgprint((char*)"relp session %p reestablished, state %d, "
 			"now resending %d unacked frames\n",
 			(void*)pThis, pThis->sessState, pThis->lenUnackedLst);
 	assert(pThis->sessState != eRelpSessState_BROKEN);
 	while(pUnackedEtry != NULL) {
-		pThis->pEngine->dbgprint("resending frame '%s'\n", pUnackedEtry->pSendbuf->pData + 9
+		pThis->pEngine->dbgprint((char*)"resending frame '%s'\n", pUnackedEtry->pSendbuf->pData + 9
 								   - pUnackedEtry->pSendbuf->lenTxnr);
 		CHKRet(relpFrameRewriteTxnr(pUnackedEtry->pSendbuf, pThis->txnr));
 		pThis->txnr = relpEngineNextTXNR(pThis->txnr);
@@ -732,7 +737,7 @@ relpSessTryReestablish(relpSess_t *pThis)
 	}
 
 finalize_it:
-	pThis->pEngine->dbgprint("after TryReestablish, sess state %d\n", pThis->sessState);
+	pThis->pEngine->dbgprint((char*)"after TryReestablish, sess state %d\n", pThis->sessState);
 	LEAVE_RELPFUNC;
 }
 
@@ -756,7 +761,7 @@ finalize_it:
  * rgerhards, 2008-03-25
  */
 static relpRetVal
-relpSessCBrspOpen(relpSess_t *pThis, relpFrame_t *pFrame)
+relpSessCBrspOpen(relpSess_t *const pThis, relpFrame_t *pFrame)
 {
 	relpEngine_t *pEngine;
 	relpOffers_t *pOffers = NULL;
@@ -775,7 +780,7 @@ relpSessCBrspOpen(relpSess_t *pThis, relpFrame_t *pFrame)
 	 * something truely unacceptable, we break the session.
 	 */
 	for(pOffer = pOffers->pRoot ; pOffer != NULL ; pOffer = pOffer->pNext) {
-		pEngine->dbgprint("processing server offer '%s'\n", pOffer->szName);
+		pEngine->dbgprint((char*)"processing server offer '%s'\n", pOffer->szName);
 		if(!strcmp((char*)pOffer->szName, "relp_version")) {
 			if(pOffer->pValueRoot == NULL)
 				ABORT_FINALIZE(RELP_RET_INVALID_OFFER);
@@ -792,7 +797,7 @@ relpSessCBrspOpen(relpSess_t *pThis, relpFrame_t *pFrame)
 			for(pOfferVal = pOffer->pValueRoot ; pOfferVal != NULL ; pOfferVal = pOfferVal->pNext) {
 				/* we do not care about return code in this case */
 				relpSessSetEnableCmd(pThis, pOfferVal->szVal, eRelpCmdState_Enabled);
-				pEngine->dbgprint("enabled command '%s'\n", pOfferVal->szVal);
+				pEngine->dbgprint((char*)"enabled command '%s'\n", pOfferVal->szVal);
 			}
 		} else if(!strcmp((char*)pOffer->szName, "relp_software")) {
 			/* we know this parameter, but we do not do anything
@@ -804,7 +809,7 @@ relpSessCBrspOpen(relpSess_t *pThis, relpFrame_t *pFrame)
 			 * case, we may simply not support it (but the client does and
 			 * must now live without it...)
 			 */
-			pEngine->dbgprint("ignoring unknown server offer '%s'\n", pOffer->szName);
+			pEngine->dbgprint((char*)"ignoring unknown server offer '%s'\n", pOffer->szName);
 		}
 	}
 
@@ -823,8 +828,8 @@ finalize_it:
  * there are some left as "Required", they are not supported by the server,
  * in which case we can not open the session. -- rgerhards, 2008-03-27
  */
-relpRetVal
-relpSessCltConnChkOffers(relpSess_t *pThis)
+static relpRetVal
+relpSessCltConnChkOffers(relpSess_t *const pThis)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -834,7 +839,7 @@ relpSessCltConnChkOffers(relpSess_t *pThis)
 
 finalize_it:
 	if(iRet != RELP_RET_OK) {
-		callOnErr(pThis, "error in CltConnChkOffers, session broken", RELP_RET_SESSION_BROKEN);
+		callOnErr(pThis, (char*) "error in CltConnChkOffers, session broken", RELP_RET_SESSION_BROKEN);
 		pThis->sessState = eRelpSessState_BROKEN;
 	}
 
@@ -847,7 +852,7 @@ finalize_it:
  * rgerhards, 2008-03-19
  */
 relpRetVal
-relpSessConnect(relpSess_t *pThis, int protFamily, unsigned char *port, unsigned char *host)
+relpSessConnect(relpSess_t *const pThis, int protFamily, unsigned char *port, unsigned char *host)
 {
 	relpOffers_t *pOffers;
 	unsigned char *pszOffers = NULL;
@@ -860,8 +865,10 @@ relpSessConnect(relpSess_t *pThis, int protFamily, unsigned char *port, unsigned
 		pThis->protFamily = protFamily;
 		if((pThis->srvPort = (unsigned char*) strdup((char*)port)) == NULL)
 			ABORT_FINALIZE(RELP_RET_OUT_OF_MEMORY);
-		if((pThis->srvAddr = (unsigned char*) strdup((char*)host)) == NULL)
-			ABORT_FINALIZE(RELP_RET_OUT_OF_MEMORY);
+		if (host != NULL) { // host can be the same pointer as pThis->srvAddr, so check for NULL!
+			if((pThis->srvAddr = (unsigned char*) strdup((char*)host)) == NULL)
+				ABORT_FINALIZE(RELP_RET_OUT_OF_MEMORY);
+		}
 	}
 
 	if (pThis->pCurrRcvFrame != NULL) {
@@ -913,10 +920,10 @@ relpSessConnect(relpSess_t *pThis, int protFamily, unsigned char *port, unsigned
 finalize_it:
 	free(pszOffers);
 	if(iRet != RELP_RET_OK) {
-		callOnErr(pThis, "error opening connection to remote peer", iRet);
+		callOnErr(pThis, (char*) "error opening connection to remote peer", iRet);
 		if((pThis->pUnackedLstLast != NULL) &&
 		   !strncmp((char*)pThis->pUnackedLstLast->pSendbuf->pData + 9, " open ", 6)) {
-			pThis->pEngine->dbgprint("relpSessConnect remove 'open' from unacked list\n");
+			pThis->pEngine->dbgprint((char*)"relpSessConnect remove 'open' from unacked list\n");
 			relpSessUnacked_t *pDel = pThis->pUnackedLstLast;
 			DLL_Del(pDel, pThis->pUnackedLstRoot, pThis->pUnackedLstLast);
 			--pThis->lenUnackedLst;
@@ -933,7 +940,7 @@ finalize_it:
  * rgerhars, 2008-03-21
  */
 static relpRetVal
-relpSessCBrspClose(relpSess_t *pThis, relpFrame_t LIBRELP_ATTR_UNUSED *pFrame)
+relpSessCBrspClose(relpSess_t *const pThis, relpFrame_t LIBRELP_ATTR_UNUSED *pFrame)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -950,7 +957,7 @@ relpSessCBrspClose(relpSess_t *pThis, relpFrame_t LIBRELP_ATTR_UNUSED *pFrame)
  * rgerhards, 2008-03-21
  */
 static relpRetVal
-relpSessCltDoDisconnect(relpSess_t *pThis)
+relpSessCltDoDisconnect(relpSess_t *const pThis)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -975,7 +982,7 @@ finalize_it:
 
 
 relpRetVal
-relpSessSetWindowSize(relpSess_t *pThis, int sizeWindow)
+relpSessSetWindowSize(relpSess_t *const pThis, int sizeWindow)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -985,7 +992,7 @@ relpSessSetWindowSize(relpSess_t *pThis, int sizeWindow)
 }
 
 relpRetVal
-relpSessSetTimeout(relpSess_t *pThis, unsigned timeout)
+relpSessSetTimeout(relpSess_t *const pThis, unsigned timeout)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -994,7 +1001,7 @@ relpSessSetTimeout(relpSess_t *pThis, unsigned timeout)
 }
 
 relpRetVal
-relpSessSetConnTimeout(relpSess_t *pThis, int connTimeout)
+relpSessSetConnTimeout(relpSess_t *const pThis, int connTimeout)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1003,7 +1010,7 @@ relpSessSetConnTimeout(relpSess_t *pThis, int connTimeout)
 }
 
 relpRetVal
-relpSessSetClientIP(relpSess_t *pThis, unsigned char *ip)
+relpSessSetClientIP(relpSess_t *const pThis, unsigned char *ip)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1019,7 +1026,7 @@ relpSessSetClientIP(relpSess_t *pThis, unsigned char *ip)
  * session object.
  */
 relpRetVal
-relpSessSetPermittedPeers(relpSess_t *pThis, relpPermittedPeers_t *pPeers)
+relpSessSetPermittedPeers(relpSess_t *const pThis, relpPermittedPeers_t *pPeers)
 {
 	ENTER_RELPFUNC;
 	int i;
@@ -1044,7 +1051,7 @@ finalize_it:
 
 /* Enable TLS mode. */
 relpRetVal
-relpSessEnableTLS(relpSess_t *pThis)
+relpSessEnableTLS(relpSess_t *const pThis)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1053,7 +1060,7 @@ relpSessEnableTLS(relpSess_t *pThis)
 }
 
 relpRetVal
-relpSessSetUsrPtr(relpSess_t *pThis, void *pUsr)
+relpSessSetUsrPtr(relpSess_t *const pThis, void *pUsr)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1061,7 +1068,7 @@ relpSessSetUsrPtr(relpSess_t *pThis, void *pUsr)
 	LEAVE_RELPFUNC;
 }
 
-relpRetVal relpSessSetMaxDataSize(relpSess_t *pThis, size_t maxSize) {
+relpRetVal relpSessSetMaxDataSize(relpSess_t *const pThis, size_t maxSize) {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
 	pThis->maxDataSize = maxSize;
@@ -1069,7 +1076,7 @@ relpRetVal relpSessSetMaxDataSize(relpSess_t *pThis, size_t maxSize) {
 }
 
 relpRetVal
-relpSessSetAuthMode(relpSess_t *pThis, relpAuthMode_t authmode)
+relpSessSetAuthMode(relpSess_t *const pThis, relpAuthMode_t authmode)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1079,7 +1086,7 @@ relpSessSetAuthMode(relpSess_t *pThis, relpAuthMode_t authmode)
 
 /* Enable TLS Zip mode. */
 relpRetVal
-relpSessEnableTLSZip(relpSess_t *pThis)
+relpSessEnableTLSZip(relpSess_t *const pThis)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1088,7 +1095,7 @@ relpSessEnableTLSZip(relpSess_t *pThis)
 }
 
 relpRetVal
-relpSessSetGnuTLSPriString(relpSess_t *pThis, char *pristr)
+relpSessSetGnuTLSPriString(relpSess_t *const pThis, char *pristr)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1105,7 +1112,7 @@ finalize_it:
 }
 
 relpRetVal
-relpSessSetCACert(relpSess_t *pThis, char *cert)
+relpSessSetCACert(relpSess_t *const pThis, char *cert)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1122,7 +1129,7 @@ finalize_it:
 }
 
 relpRetVal
-relpSessSetOwnCert(relpSess_t *pThis, char *cert)
+relpSessSetOwnCert(relpSess_t *const pThis, char *cert)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1139,7 +1146,7 @@ finalize_it:
 }
 
 relpRetVal
-relpSessSetPrivKey(relpSess_t *pThis, char *cert)
+relpSessSetPrivKey(relpSess_t *const pThis, char *cert)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1156,7 +1163,7 @@ finalize_it:
 }
 
 relpRetVal
-relpSessSetTlsConfigCmd(relpSess_t *pThis, char *cfgcmd)
+relpSessSetTlsConfigCmd(relpSess_t *const pThis, char *cfgcmd)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1177,7 +1184,7 @@ finalize_it:
  * rgerhards, 2008-03-25
  */
 relpRetVal
-relpSessSetProtocolVersion(relpSess_t *pThis, int protocolVersion)
+relpSessSetProtocolVersion(relpSess_t *const pThis, int protocolVersion)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1192,7 +1199,7 @@ relpSessSetProtocolVersion(relpSess_t *pThis, int protocolVersion)
  * rgerhards, 2008-03-25
  */
 relpRetVal
-relpSessSetEnableCmd(relpSess_t *pThis, unsigned char *pszCmd, relpCmdEnaState_t stateCmd)
+relpSessSetEnableCmd(relpSess_t *const pThis, unsigned char *pszCmd, relpCmdEnaState_t stateCmd)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1202,7 +1209,7 @@ relpSessSetEnableCmd(relpSess_t *pThis, unsigned char *pszCmd, relpCmdEnaState_t
 		if(pThis->stateCmdSyslog != eRelpCmdState_Forbidden)
 			pThis->stateCmdSyslog = stateCmd;
 	} else {
-		pThis->pEngine->dbgprint("tried to set unknown command '%s' to %d\n", pszCmd, stateCmd);
+		pThis->pEngine->dbgprint((char*)"tried to set unknown command '%s' to %d\n", pszCmd, stateCmd);
 		ABORT_FINALIZE(RELP_RET_UNKNOWN_CMD);
 	}
 
@@ -1215,7 +1222,7 @@ finalize_it:
  * rgerhards, 2008-03-25
  */
 relpRetVal
-relpSessConstructOffers(relpSess_t *pThis, relpOffers_t **ppOffers)
+relpSessConstructOffers(relpSess_t *const pThis, relpOffers_t **ppOffers)
 {
 	relpOffers_t *pOffers;
 	relpOffer_t *pOffer;
@@ -1265,7 +1272,7 @@ finalize_it:
  * rgerhards, 2008-03-25
  */
 static relpRetVal
-relpSessFixCmdStates(relpSess_t *pThis)
+relpSessFixCmdStates(relpSess_t *const pThis)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Sess);
@@ -1285,7 +1292,7 @@ relpSessFixCmdStates(relpSess_t *pThis)
  * rgerhards, 2008-03-25
  */
 relpRetVal
-relpSessSendSyslog(relpSess_t *pThis, unsigned char *pMsg, size_t lenMsg)
+relpSessSendSyslog(relpSess_t *const pThis, unsigned char *pMsg, const size_t lenMsg)
 {
 
 	ENTER_RELPFUNC;
