@@ -141,6 +141,9 @@ typedef struct relpTcp_s {
 	osslSslState_t sslState;/**< what must we retry? */
 	#endif
 	relpTcpRtryState_t rtryOp;
+	#ifdef ENABLE_TLS_OPENSSL
+	int rtryOsslErr;	/**< store optional ssl error code into like SSL_ERROR_WANT_READ or SSL_ERROR_WANT_WRITE */
+	#endif
 } relpTcp_t;
 
 /* macros for quick member access */
@@ -182,6 +185,7 @@ void relpTcpHintBurstBegin(relpTcp_t *pThis);
 void relpTcpHintBurstEnd(relpTcp_t *pThis);
 int LIBRELP_ATTR_NONNULL() relpTcpGetRtryDirection(relpTcp_t *pThis);
 int relpTcpWaitWriteable(relpTcp_t *pThis, struct timespec *timeout);
+void relpTcpExitTLS(void);
 
 #ifdef ENABLE_TLS_OPENSSL
 /*-----------------------------------------------------------------------------*/
@@ -207,12 +211,13 @@ struct CRYPTO_dynlock_value * dyn_create_function(
 unsigned long id_function(void);
 void locking_function(int mode, int n,
 	__attribute__((unused)) const char * file, __attribute__((unused)) int line);
-/*-----------------------------------------------------------------------------*/
 
-/* Helper to detect when OpenSSL is initialized */
-static int called_openssl_global_init = 0;
-/* Main OpenSSL CTX pointer */
-static SSL_CTX *ctx;
+int opensslh_THREAD_setup(void);
+int opensslh_THREAD_cleanup(void);
+void relpTcpLastSSLErrorMsg(int ret, relpTcp_t *const pThis, const char* pszCallSource);
+int verify_callback(int status, X509_STORE_CTX *store);
+
+/*-----------------------------------------------------------------------------*/
 #endif
 
 #endif /* #ifndef RELPTCP_H_INCLUDED */
