@@ -62,7 +62,7 @@ hdlr_enable(int sig, void (*hdlr)())
 	sigaction(sig, &sigAct, NULL);
 }
 /* handler for unexpected signals.  */
-void LIBRELP_ATTR_NORETURN
+static void LIBRELP_ATTR_NORETURN
 do_signal(const int sig)
 {
 	fprintf(stderr, "send: UNEXPECTED SIGNAL %d%s- terminating\n", sig,
@@ -83,7 +83,7 @@ dbgprintf(char *fmt, ...)
 	fprintf(dbgFile, "send.c: %s", pszWriteBuf);
 }
 
-void print_usage(void)
+static void print_usage(void)
 {
 	printf("Usage: send -t <SERVER> -p <PORTNUM> -m <MESSAGE>\n");
 }
@@ -92,14 +92,19 @@ static void
 onErr(void *pUsr, char *objinfo, char* errmesg, LIBRELP_ATTR_UNUSED relpRetVal errcode)
 {
 	struct usrdata *pThis = (struct usrdata*) pUsr;
-	if(pUsr != userdata) {
-		fprintf(stderr, "send: pUsr NOT pointing to usrdata!\n");
+	if(pUsr != NULL) {
+		if(pUsr != userdata) {
+			fprintf(stderr, "send: pUsr %p NOT pointing to userdata %p!\n", pUsr, (void*)userdata);
+		}
+		if(pThis->magic != USR_MAGIC) {
+			fprintf(stderr, "send: pUsr magic incorrect in onErr, magic %8.8x "
+				"pUsr %p\n", pThis->magic, (void*) pThis);
+		}
+		fprintf(stderr, "%s: error '%s', object '%s'\n", pThis->progname, errmesg, objinfo);
+	} else {
+		fprintf(stderr, "send: [pUsr==NULL] error '%s', object '%s'\n", errmesg, objinfo);
 	}
-	if(pThis->magic != USR_MAGIC) {
-		fprintf(stderr, "send: pUsr magic incorrect in onErr, magic %8.8x "
-			"pUsr %p\n", pThis->magic, (void*) pThis);
-	}
-	printf("%s: error '%s', object '%s'\n", pThis->progname, errmesg, objinfo);
+
 	if(errFile != NULL) {
 		fprintf(errFile, "send: error '%s', object '%s'\n", errmesg, objinfo);
 	}
@@ -108,7 +113,7 @@ onErr(void *pUsr, char *objinfo, char* errmesg, LIBRELP_ATTR_UNUSED relpRetVal e
 static void
 onGenericErr(char *objinfo, char* errmesg, LIBRELP_ATTR_UNUSED relpRetVal errcode)
 {
-	printf("send: librelp error '%s', object '%s'\n", errmesg, objinfo);
+	fprintf(stderr, "send: librelp error '%s', object '%s'\n", errmesg, objinfo);
 	if(errFile != NULL) {
 		fprintf(errFile, "send: librelp error '%s', object '%s'\n", errmesg, objinfo);
 	}
@@ -119,7 +124,7 @@ static void
 onAuthErr( LIBRELP_ATTR_UNUSED void *pUsr, char *authinfo,
 	char* errmesg, LIBRELP_ATTR_UNUSED relpRetVal errcode)
 {
-	printf("send: authentication error '%s', object '%s'\n", errmesg, authinfo);
+	fprintf(stderr, "send: authentication error '%s', object '%s'\n", errmesg, authinfo);
 	if(errFile != NULL) {
 		fprintf(errFile, "send: authentication error '%s', object '%s'\n", errmesg, authinfo);
 	}
@@ -200,7 +205,7 @@ send_msgs_counter(void)
 static int
 send_msgs_single(const char *pMsg)
 {
-	int ret;
+	int ret = 0;
 	char *msgData = NULL;
 
 	if(msgDataLen != 0) {
@@ -218,7 +223,6 @@ send_msgs_single(const char *pMsg)
 	}
 
 	TRY(relpCltSendSyslog(pRelpClt, (unsigned char *)pMsg, lenMsg));
-
 	if(msgDataLen != 0) {
 		free((void *)pMsg);
 	}
@@ -237,6 +241,7 @@ int main(int argc, char *argv[]) {
 	int verbose = 0;
 	int protFamily = 2; /* IPv4=2, IPv6=10 */
 	int bEnableTLS = 0;
+	int no_exit_on_err = 0;
 	char *caCertFile = NULL;
 	char *myCertFile = NULL;
 	char *myPrivKeyFile = NULL;
@@ -271,6 +276,7 @@ int main(int argc, char *argv[]) {
 		{"kill-signal", required_argument, 0, KILL_SIGNAL},
 		{"kill-pid", required_argument, 0, KILL_PID},
 		{"connect-retries", required_argument, 0, CONNECT_RETRIES},
+		{"no-exit-on-error", no_argument, 0, 'N'},
 		{0, 0, 0, 0}
 	};
 
@@ -340,6 +346,9 @@ int main(int argc, char *argv[]) {
 		case 'z':
 			myPrivKeyFile = optarg;
 			break;
+		case 'N':
+			no_exit_on_err = 1;
+			break;
 		case KILL_ON_MSG:
 			kill_on_msg = atoi(optarg);
 			break;
@@ -359,7 +368,11 @@ int main(int argc, char *argv[]) {
 	}
 
 	atexit(exit_hdlr);
-	hdlr_enable(SIGPIPE, do_signal);
+	if (no_exit_on_err == 0) {
+		hdlr_enable(SIGPIPE, do_signal);
+	} else {
+		signal(SIGPIPE, SIG_IGN);
+	}
 
 	if(msgDataLen != 0 && msgDataLen < lenMsg) {
 		fprintf(stderr, "send: message is larger than configured message size!\n");
@@ -374,7 +387,7 @@ int main(int argc, char *argv[]) {
 
 	if(authMode != NULL) {
 		if(	(strcasecmp(authMode, "certvalid")  != 0 && permittedPeer == NULL) ||
-			caCertFile == NULL || myCertFile == NULL || myPrivKeyFile == NULL) {
+			myCertFile == NULL || myPrivKeyFile == NULL) {
 			printf("send: mode '%s' parameter missing; certificates and permittedPeer required\n",
 				authMode);
 			exit(1);
@@ -399,23 +412,24 @@ int main(int argc, char *argv[]) {
 
 	TRY(relpEngineConstruct(&pRelpEngine));
 	TRY(relpEngineSetDbgprint(pRelpEngine, verbose ? dbgprintf : NULL));
-	if(tlslib != NULL) {
-		TRY(relpEngineSetTLSLibByName(pRelpEngine, tlslib));
-	}
-
 	TRY(relpEngineSetOnErr(pRelpEngine, onErr));
 	TRY(relpEngineSetOnGenericErr(pRelpEngine, onGenericErr));
 	TRY(relpEngineSetOnAuthErr(pRelpEngine, onAuthErr));
 
+	if(tlslib != NULL) {
+		TRY(relpEngineSetTLSLibByName(pRelpEngine, tlslib));
+	}
+
 
 	TRY(relpEngineSetEnableCmd(pRelpEngine, (unsigned char*)"syslog", eRelpCmdState_Required));
 	TRY(relpEngineCltConstruct(pRelpEngine, &pRelpClt));
-	TRY(relpCltSetTimeout(pRelpClt, timeout));
+	// Create userdata pointer as soon as possible for error callbacks
 	userdata = calloc(1, sizeof(struct usrdata));
 	userdata->magic = USR_MAGIC;
 	userdata->progname = strdup("send");
 	TRY(relpCltSetUsrPtr(pRelpClt, userdata));
 
+	TRY(relpCltSetTimeout(pRelpClt, timeout));
 	if(bEnableTLS) {
 		TRY(relpCltEnableTLS(pRelpClt));
 		TRY(relpCltSetTlsConfigCmd(pRelpClt, tlsConfigCmd));
@@ -428,9 +442,16 @@ int main(int argc, char *argv[]) {
 				TRY(relpCltAddPermittedPeer(pRelpClt, permittedPeer));
 			}
 		}
+		printf("send: Init TLS Parameters DONE\n");
 	}
-
-	TRY(relpCltConnect(pRelpClt, protFamily, port, target));
+	// Need to check error code!
+	ret = relpCltConnect(pRelpClt, protFamily, port, target); 
+	if(ret != RELP_RET_OK) {
+		fprintf(stderr, "send: FAILURE in 'relpCltConnect' with errorcode %d\n", ret);
+		ret = 1; 
+		goto done;
+	}
+	printf("send: Connect DONE\n");
 
 	if(num_messages == 0) {
 		send_msgs_single(pMsg);
