@@ -8,13 +8,13 @@ export NUMMESSAGES=100000
 
 actual_test() {
 	startup_receiver
-	./send -t 127.0.0.1 -p $TESTPORT -n$NUMMESSAGES \
+	./send -t 127.0.0.1 -p $TESTPORT -n$NUMMESSAGES --no-exit-on-error \
 		--kill-on-msg 20000 --kill-pid $RECEIVE_PID $OPT_VERBOSE &
 	SENDER_PID=$!
 
 	for i in {1..3};  do
 		sleep 2
-		timeout 10s ${srcdir}/dummyserver.py $TESTPORT
+		timeout 10s $PYTHON ${srcdir}/dummyserver.py $TESTPORT
 	done
 
 	sleep 2 # make sure client goes into retry
